<?php
// $Id: ip_authenticator.admin.inc,v 1.2 2008/05/08 12:40:28 jonfrancisskydiver Exp $
/*
 * IP based authenticator
 * @author Jonathan T. Francis
 *
 */
drupal_add_css(drupal_get_path("module", "ip_authenticator") ."/css/ip_authenticator.admin.css", "module", "all", FALSE);
drupal_add_js( drupal_get_path("module", "ip_authenticator") ."/js/ip_authenticator.admin.js");


/**
 * administer the ip_authenticator parameters.
 */
function ip_authenticator_admin($form_state) {
  global $user;
  
  $form['ip_list'] = array(
    '#type' => 'fieldset',
    '#title' => t('Current IP Authenticators') ."<br/>",
    '#description' => t('Click on a link to modify the authenticator assignment'),
    '#collapsible' => FALSE,
    '#callapsed' => FALSE,
    '#weight' => 0    
  );  
  
  $roles = user_roles();
  $result = _get_ip_authenticators(null, "id, roles, ip1, ip2");
  $i=0;
  while ($row = db_fetch_array($result)) {
    
    $row["ip1"] = long2ip($row["ip1"]); //this takes the mysql command, INET_NTOA out of the query string.
    
    if (0 == $row["ip2"]) 
      $row["ip2"] = "";
    else 
      $row["ip2"] = long2ip($row["ip2"]);
    
    $ip2_text = ($row["ip2"]!="") ? " - " . $row["ip2"] : "";
    $title = $row['ip1'] . $ip2_text ." authenticates to: ". $roles[$row["roles"]];
    
    $form['ip_list'][$row["id"]] = array(
      '#prefix' => '<div>',
      '#suffix' => '</div>',
      '#value' => l($title, "admin/settings/ip_authenticator/edit/". $row["id"])
    );
  }
  
  $form['ips'] = array(
    '#type' => 'fieldset',
    '#title' => t('IP address assignment') ."<br/>",
    '#collapsible' => FALSE,
    '#callapsed' => FALSE,
    '#weight' => 0
  );
  
  $form['ips']['ip1'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'textfield',
    '#title' => 'IP 1',
    '#size' => '15',
    '#required' => TRUE,
    '#suffix' => '</div>'
  );
  $form['ips']['seperator'] = array(
    '#prefix' => '<div class="floatLeft" id="formSeperator">',
    '#value' => '&nbsp;&nbsp; - &nbsp;&nbsp;',
    '#suffix' => '</div>',
  );
  $form['ips']['ip2'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'textfield',
    '#title' => 'IP 2',
    '#description' => 'Leave blank for<br/>individual IPs',
    '#size' => '15',
    '#suffix' => '</div>',
  );
  $form['ips']['seperator1'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#value' => '&nbsp; &nbsp; &nbsp; &nbsp;',
    '#suffix' => '</div>',    
  );  
  $form['ips']['role'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'select',
    '#title' => t('Assign to Role'),
    '#options' => $roles,
    '#description' => "Select a role to assign the ips",
    '#suffix' => '</div>',
  );
  
  $form['#validate'][] = 'ip_authenticator_validate';
  $form['#submit'][] = 'ip_authenticator_submit';
  
  return system_settings_form($form);
}

function ip_authenticator_validate($form, &$form_state) {

  if (!preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/x", $form_state['values']['ip1'])) {
    form_set_error('ip1', "You must enter an ip address in the first field.");
  }
  
  if ("" != trim($form_values['values']['ip2']) && !preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/x", trim($form_values['values']['ip2'])) ) {
    form_set_error('ip2', "You must enter a valid IP address in the second field, or leave the second field blank.");
  }
  
}

function ip_authenticator_submit($form, &$form_values) {
  if (!preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/x", trim($form_values['values']['ip2'])) ) 
    $ip2 = trim($form_values["values"]["ip2"]);
  else
    $ip2 = ip2long(trim($form_values["values"]['ip2']));
    
  $sql = "INSERT INTO {ip_authenticator} (ip1, ip2, roles) VALUES ('%s','%s','%s')";
  db_query($sql, ip2long(trim($form_values['values']['ip1'])), $ip2, $form_values['values']['role']);
  if (db_affected_rows() == 0) {
    echo mysql_error();
    exit;
  }
}

function _get_ip_role_info($id) {
  
  $result = db_query("SELECT roles, ip1, ip2 FROM {ip_authenticator} WHERE id = '%s'", $id);
  $row = db_fetch_array($result);
  $row["ip1"] = long2ip($row["ip1"]);
  
  if ($row["ip2"] == 0) 
    $row["ip2"] = "";
  else 
    $row["ip2"] = long2ip($row["ip2"]); 

  return $row;  
}

function ip_authenticator_admin_edit($arg, $arg1, $arg3) {
  
  if (!preg_match("/^[0-9]+\$/", $arg3)) {
    drupal_set_message("Please select your authenticator again");
    header("Location: /admin/settings/ip_authenticator");
    exit;
  }
  
  $row = _get_ip_role_info($arg3);
  
  $form['ips'] = array(
    '#type' => 'fieldset',
    '#title' => t('IP address assignment'),
    '#collapsible' => FALSE,
    '#callapsed' => FALSE,
    '#weight' => 0
  );
  
  $form['ips']['ip1'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'textfield',
    '#title' => 'IP 1',
    '#size' => '15',
    '#default_value' => $row["ip1"],
    '#required' => TRUE,
    '#suffix' => '</div>'
  );
  $form['ips']['seperator'] = array(
    '#prefix' => '<div class="floatLeft" id="formSeperator">',
    '#value' => '&nbsp;&nbsp; - &nbsp;&nbsp;',
    '#suffix' => '</div>',
  );
  $form['ips']['ip2'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'textfield',
    '#title' => 'IP 2',
    '#description' => 'Leave blank for<br/>individual IPs',
    '#size' => '15',
    '#default_value' => $row["ip2"],
    '#suffix' => '</div>',
  );
  $form['ips']['seperator1'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#value' => '&nbsp; &nbsp; &nbsp; &nbsp;',
    '#suffix' => '</div>',    
  );
  $form['ips']['role'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#type' => 'select',
    '#title' => t('Assign to Role'),
    '#options' => user_roles(),
    '#default_value' => $row["roles"],
    '#description' => "Select a role to assign the ips",
    '#suffix' => '</div>',
  );
  $form['ips']['delete'] = array(
    '#prefix' => '<div class="floatLeft" id="deleteLink">',
    '#value' => '&nbsp; &nbsp; &nbsp; &nbsp;'. l("delete", "admin/settings/ip_authenticator/delete/" . $arg3),
    '#suffix' => '</div>',
  );
  $form['id'] = array(
    '#type' => 'hidden',
    '#value' => $arg3   
  );
  $form['#validate'][] = 'ip_authenticator_validate';
  $form['#submit'][] = 'ip_authenticator_edit_submit';
  $form['#redirect'] = 'admin/settings/ip_authenticator';
  
  return system_settings_form($form); 
}


function ip_authenticator_edit_submit($form, &$form_values) {
  if (!preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/x", trim($form_values['values']['ip2'])) ) 
    $ip2_long = trim($form_values['values']['ip2']);
  else
    $ip2_long = trim(ip2long($form_values['values']['ip2']));
  
  $sql = "UPDATE {ip_authenticator} SET ip1 = '%s', ip2 = '%s', roles = '%s' WHERE id = '%s'";  
  
  db_query($sql, ip2long(trim($form_values['values']['ip1'])), $ip2_long, $form_values['values']['role'], $form_values['values']['id']);
  if (db_affected_rows() == 0) {
    echo mysql_error();
    exit;
  }
}


function ip_authenticator_admin_delete($arg, $arg1, $arg3) {  
  
  if (!preg_match("/^[0-9]+\$/", $arg3)) {
    drupal_set_message("Please select your authenticator again");
    header("Location: /admin/settings/ip_authenticator");
    exit;
  }
  
  $row = _get_ip_role_info($arg3);
  $roles = user_roles();
  
  $ip2_text = ($row["ip2"] != "") ? " - ". $row["ip2"] : "";
  $title = "<li>" . $row['ip1'] . $ip2_text ." authenticates to: ". $roles[$row["roles"]];

  $form["markup"] = array(
    '#prefix' => "<div id='confirmbox'>",
    '#suffix' => "",
    '#value' => "Are you certain that you wish to delete this ip authenticator? <ul>". $title ."</ul>"    
  );
  
  $form["accept"] = array(
    '#type' => 'submit',
    '#value' => 'Delete',
    '#prefix' => '<div class="floatLeft">',
    '#suffix' => '</div>'
  );
  $form['seperator1'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#value' => '&nbsp; &nbsp; &nbsp; &nbsp;',
    '#suffix' => '</div>',  
  );
  $form['cancel'] = array(
    '#prefix' => '<div class="floatLeft">',
    '#value' => '<input type="button" value="Cancel" id="cancel">',
    '#suffix' => '</div></div>'
  );
  $form['id'] = array(
    '#type' => 'hidden', 
    '#value' => $arg3   
  );
  
  $form['#submit'][] = 'ip_authenticator_admin_delete_submit';
  $form['#redirect'] = 'admin/settings/ip_authenticator';
  $form['#validate'][] = 'ip_authentiator_admin_delete_validate';
  return $form;
}

function ip_authentiator_admin_delete_validate($form, &$form_values) {
  if (!preg_match("/^[0-9]+\$/", $form_values["values"]["id"]))
    form_set_error("id", "Please return to the ". l("admin/settings/ip_authenticator", "ip_authenticator") ." page and then navigate back here. A form value was lost.");
}

function ip_authenticator_admin_delete_submit($form, &$form_values) {
  $result = db_query("DELETE FROM {ip_authenticator} WHERE id = '%s'", $form_values['values']['id']);
  if (db_affected_rows() < 1) {
    drupal_set_message("Error in execution of sql statement -- no changes made.");
  }
  else {
    drupal_set_message("The IP Authenticator item has been deleted.");
  }
}